==> blobAndConquer-0.1-1.tar.gz: BlobAndConquer/src/bioMechBlobs.cpp <==
/*
Copyright (C) 2005 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "bioMechBlobs.h"

// ########################### General Stuff ################################

void generalMechDeathBoom()
{
	Unit *unit = (Unit*)self;

	if (unit->health > -100)
	{
		unit->health -= engine->getTimeDifference(TD_LOGIC);
		return;
	}

	Math::removeBit(&unit->flags, EF_DYING);
	Math::addBit(&unit->flags, EF_DEAD);

	doExplosion(unit->position, 10, self);

	for (int i = 0 ; i < 10 ; i++)
	{
		throwExplosionDebris(self->position);
	}

	self->action = NULL;
	self->velocity.set(0, 0, 0);
}

void generalBlobDeathSplat()
{
	Unit *unit = (Unit*)self;
	
	if (unit->health > -100)
	{
		unit->health -= engine->getTimeDifference(TD_LOGIC);
		return;
	}
	
	if ((!(unit->flags & EF_ONGROUND)) && (unit->contentType == (CONTENTS_SOLID|CONTENTS_AIR)))
	{
		unit->helpless = 25;
		Math::removeBit(&unit->flags, EF_BOUNCES);
		return;
	}

	Math::removeBit(&self->flags, EF_DYING);
	Math::addBit(&self->flags, EF_DEAD);

	if (unit->contentType & CONTENTS_SOLID)
	{
		audio->playSound(SND_SPLAT, (self == player) ? CH_PLAYER : CH_ANY);
	}
	else
	{
		audio->playSound(SND_SPLAT_UNDERWATER, (self == player) ? CH_PLAYER : CH_ANY);
	}

	for (int i = 0 ; i < 25 ; i++)
	{
		throwFleshChunk(self->position, (unit->contentType & CONTENTS_SOLID) == 0);
	}

	self->action = NULL;
	self->velocity.set(0, 0, 0);
}

void generalEnemyLookAround()
{
	Unit *unit = (Unit*)self;
	
	/*
	if (Math::getDistance(unit->lastTargetPosition, unit->position) > 2)
	{
		// haven't gone anywhere!
		if (Math::getDistance(self->oldPosition, unit->position) < 2)
		{
			self->velocity.x = Math::rrand(-20, 20);
			self->velocity.y = Math::rrand(-20, 20);
			self->velocity.x /= 35;
			self->velocity.y /= 35;
			self->thinkTime = Math::rrand(25, 150);

			angle = atan2(self->velocity.y, self->velocity.x);
			self->rotation.x = 90 + toDegrees(angle);
			
			return;
		}
		
		unit->velocity = unit->lastTargetPosition;
		unit->velocity -= unit->position;
		unit->velocity.normalize();
		
		unit->velocity.x /= 2;
		unit->velocity.y /= 2;
		
		if (!(unit->flags & EF_WEIGHTLESS))
		{
			unit->velocity.z = 0;
		}
		
		angle = atan2(self->velocity.y, self->velocity.x);
		self->rotation.x = 90 + toDegrees(angle);
		
		self->thinkTime = 25;
		
		self->oldPosition = self->position;
		
		return;
	}
	*/
	
	unit->lastTargetPosition = self->position;
	
	Vector target;

	switch (rand() % 25)
	{
		case 0:
			target = self->position;
			target.x += Math::rrand(-50, 50);
			target.y += Math::rrand(-50, 50);
			
			self->velocity = target;
			self->velocity -= self->position;
			self->velocity.normalize();
			self->velocity = self->velocity * 0.25;
			
			faceLocation((Unit*)self, target);
			
			self->thinkTime = Math::rrand(300, 500);

			break;

		case 1:
			self->velocity.x = 0;
			self->velocity.y = 0;
			self->rotation.x = Math::rrand(0, 360);
			self->thinkTime = Math::rrand(100, 300);
			break;

		default:
			self->thinkTime = Math::rrand(1, 10);
			break;
	}
}

void bioMechSearch()
{
	Unit *unit = (Unit*)self;
	
	if (unit->target != NULL)
	{
		unit->lastTargetPosition = unit->target->position;
	}
	
	if (unit->flags & EF_NOTARGET)
	{
		generalEnemyLookAround();
		return;
	}

	switch (rand() % 6)
	{
		// initially spot target
		case 0:
		case 1:
			self->velocity.x = 0;
			self->velocity.y = 0;
			if (enemySightTarget(unit, 850))
			{
				enemyMoveToTarget(unit);
				self->thinkTime = Math::rrand(50, 150);
				faceTarget(unit);
			}
			else
			{
				generalEnemyLookAround();
			}
			break;

		// strafe at target
		case 2:
			if (enemySightTarget(unit, 200))
			{
				self->action = self->attack;
				if ((rand() % 3) > 0)
				{
					self->velocity.x = 0;
					self->velocity.y = 0;
				}
				faceTarget(unit);
			}
			else
			{
				generalEnemyLookAround();
			}
			break;

		// stop and attack enemy
		case 3:
		case 4:
		case 5:
			self->velocity.x = 0;
			self->velocity.y = 0;
			self->thinkTime = Math::rrand(0, 20);
			faceTarget(unit);

			if ((rand() % 3) == 0)
			{
				if (enemySightTarget(unit, 200))
				{
					self->action = self->attack;
				}
				else
				{
					generalEnemyLookAround();
				}
			}
			break;
	}
}

void bioMechReload()
{
	audio->playSound(SND_PISTOL_RELOAD, CH_ANY);
	self->thinkTime = 100;
	self->action = &bioMechSearch;
}

WeaponDef *getRandomPlayerAmmoType()
{
	if ((rand() % 3) == 0)
	{
		return NULL;
	}
	
	if ((game->weapon[1] == NULL) && (game->weapon[2] == NULL))
	{
		return NULL;
	}
	else if ((game->weapon[1] != NULL) && (game->weapon[2] != NULL))
	{
		return (WeaponDef*)game->weapon[1 + rand() % 2]->definition;
	}
	else
	{
		return (WeaponDef*)game->weapon[1]->definition;
	}

	return NULL;
}

void generalUnitDropWeaponAndItems()
{
	Unit *unit = (Unit*)self;

	if (!(unit->flags & EF_EXPLODES))
	{
		unit->action = &generalBlobDeathSplat;
	}
	else
	{
		unit->action = &generalMechDeathBoom;
	}

	if (unit->dead)
	{
		return;
	}

	if ((unit->currentWeapon->type == WEP_PISTOL) || (unit->currentWeapon->type == WEP_GRENADE))
	{
		unit->currentWeapon = getRandomPlayerAmmoType();
	}

	if (unit->currentWeapon != NULL)
	{
		Properties weaponProps;
		
		weaponProps.setProperty("classname", unit->currentWeapon->type);
		weaponProps.setProperty("position", unit->position);
		
		Weapon *weapon = addWeapon(&weaponProps);
		WeaponDef *weaponDef = (WeaponDef*)weapon->definition;
		
		int minAmmo = (int)max(1, (weaponDef->clipSize * 0.10));
		int maxAmmo = (int)max(2, (weaponDef->clipSize * 0.15));
		
		weapon->currentAmmo = Math::rrand(minAmmo, maxAmmo);

		throwSpawnedItem(weapon);
		
		if ((rand() % 12) == 0)
		{
			weaponProps.setProperty("classname", "WEP_GRENADE");
			
			weapon = addWeapon(&weaponProps);
			weapon->currentAmmo = 1;
			
			throwSpawnedItem(weapon);
		}
	}

	unit->currentWeapon = NULL;

	if (unit->currentItem != NULL)
	{
		/*
		unit->currentItem->position = unit->position;

		throwSpawnedItem(unit->currentItem);
		*/

		unit->currentItem = NULL;
	}

	if ((rand() % 3) == 0)
	{
		Properties itemProps;
		
		itemProps.setProperty("classname", "ITM_CHERRY");
		itemProps.setProperty("position", unit->position);
		
		Item *item = addItem(&itemProps);
		item->itemPower = 1 + rand() % item->itemPower;

		throwSpawnedItem(item);
	}
}

void generalBlobDie()
{
	Unit *unit = (Unit*)self;
	
	unit->health = 0;
	unit->helpless = 200;
	
	if (unit->contentType & CONTENTS_SOLID)
	{
		self->velocity.z = Math::rrand(50, 100);
		self->velocity.z /= 100;
	}
	
	unit->thinkTime = 0;

	if (unit->liquidLevel != 2)
	{
		audio->playSound(Math::rrand(SND_DEATH1, SND_DEATH4), (unit == player) ? CH_PLAYER : CH_DEATH);
	}
	else if (unit->oxygen != 0)
	{
		audio->playSound(SND_DROWNING, (unit == player) ? CH_PLAYER : CH_DEATH);
	}

	generalUnitDropWeaponAndItems();

	Math::removeBit(&unit->flags, EF_WEIGHTLESS);
}

// ########################### Machine Gun Blob Stuff ################################

void machineGunBlobAttack()
{
	//debug(("machineGunBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &machineGunBlobAttack;

	attackTargetScatter(unit);

	if (--self->custom == 0)
	{
		self->action = &bioMechSearch;
		self->thinkTime = Math::rrand(25, 75);

		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void machineGunBlobPreAttack()
{
	//debug(("machineGunBlobPreAttack()\n"));

	self->custom = Math::rrand(3, 12);
	self->thinkTime = 0;
	self->action = &machineGunBlobAttack;
}

void machineGunBlobInit(Unit *unit)
{
	//debug(("machineGunBlobInit()\n"));

	unit->walk = &bioMechSearch;
	unit->attack = &machineGunBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ########################### Pistol Blob Stuff ################################

void pistolBlobAttack()
{
	//debug(("pistolBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &pistolBlobAttack;

	attackTarget(unit);

	if (--self->custom == 0)
	{
		self->action = &bioMechSearch;
		self->thinkTime = Math::rrand(10, 50);
		
		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void pistolBlobPreAttack()
{
	//debug(("pistolBlobPreAttack()\n"));

	self->custom = Math::rrand(1, 2);
	self->thinkTime = 0;
	self->action = &pistolBlobAttack;
}

void pistolBlobInit(Unit *unit)
{
	//debug(("pistolBlobInit()\n"));

	unit->walk = &bioMechSearch;
	unit->attack = &pistolBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ########################### Shotgun Blob Stuff ################################

void shotGunBlobAttack()
{
	//debug(("shotGunBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &shotGunBlobAttack;

	attackTarget(unit);

	if (--self->custom == 0)
	{
		self->action = &bioMechSearch;
		self->thinkTime = Math::rrand(25, 75);
		
		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void shotGunBlobPreAttack()
{
	self->custom = 1;
	self->thinkTime = 0;
	self->action = &shotGunBlobAttack;
}

void shotGunBlobInit(Unit *unit)
{
	unit->walk = &bioMechSearch;
	unit->attack = &shotGunBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ############################ Grenade Blob Stuff ###################################

void grenadeBlobAttack()
{
	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	attackTarget(unit);

	if (--unit->custom == 0)
	{
		self->thinkTime = Math::rrand(100, 200);
		self->action = &bioMechSearch;
		
		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void grenadeBlobPreAttack()
{
	if ((rand() % 4) > 1)
	{
		((Unit*)self)->currentWeapon = &entityManager->weaponDef[WEP_GRENADE];
		self->custom = Math::rrand(1, 3);
		self->action = grenadeBlobAttack;
	}
	else
	{
		((Unit*)self)->currentWeapon = &entityManager->weaponDef[WEP_PISTOL];
		self->custom = Math::rrand(1, 2);
		self->action = pistolBlobAttack;
	}
}

void grenadeBlobInit(Unit *unit)
{
	unit->walk = &bioMechSearch;
	unit->attack = &grenadeBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

==> blobAndConquer-0.2-1.tar.gz: BlobAndConquer/src/bioMechBlobs.cpp <==
/*
Copyright (C) 2006 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "bioMechBlobs.h"

// ########################### General Stuff ################################

void generalMechDeathBoom()
{
	Unit *unit = (Unit*)self;

	if (unit->health > -100)
	{
		unit->health -= engine->getTimeDifference(TD_LOGIC);
		return;
	}

	Math::removeBit(&unit->flags, EF_DYING);
	Math::addBit(&unit->flags, EF_DEAD);

	doExplosion(unit->position, 10, self);

	for (int i = 0 ; i < 10 ; i++)
	{
		throwExplosionDebris(self->position);
	}

	self->action = NULL;
	self->velocity.set(0, 0, 0);
}

void generalBlobDeathSplat()
{
	Unit *unit = (Unit*)self;
	
	if (unit->health > -100)
	{
		unit->health -= engine->getTimeDifference(TD_LOGIC);
		return;
	}
	
	if ((!(unit->flags & EF_ONGROUND)) && (unit->contentType == (CONTENTS_SOLID|CONTENTS_AIR)))
	{
		unit->helpless = 25;
		Math::removeBit(&unit->flags, EF_BOUNCES);
		return;
	}

	Math::removeBit(&self->flags, EF_DYING);
	Math::addBit(&self->flags, EF_DEAD);

	if (unit->liquidLevel < 2)
	{
		audio->playSound(SND_SPLAT, (self == player) ? CH_PLAYER : CH_ANY);
	}
	else
	{
		audio->playSound(SND_SPLAT_UNDERWATER, (self == player) ? CH_PLAYER : CH_ANY);
	}

	for (int i = 0 ; i < 25 ; i++)
	{
		throwFleshChunk(self->position, (unit->liquidLevel == 2));
	}

	self->action = NULL;
	self->velocity.set(0, 0, 0);
}

void generalEnemyLookAround()
{
	Unit *unit = (Unit*)self;
	
	/*
	if (Math::getDistance(unit->lastTargetPosition, unit->position) > 2)
	{
		// haven't gone anywhere!
		if (Math::getDistance(self->oldPosition, unit->position) < 2)
		{
			self->velocity.x = Math::rrand(-20, 20);
			self->velocity.y = Math::rrand(-20, 20);
			self->velocity.x /= 35;
			self->velocity.y /= 35;
			self->thinkTime = Math::rrand(25, 150);

			angle = atan2(self->velocity.y, self->velocity.x);
			self->rotation.x = 90 + toDegrees(angle);
			
			return;
		}
		
		unit->velocity = unit->lastTargetPosition;
		unit->velocity -= unit->position;
		unit->velocity.normalize();
		
		unit->velocity.x /= 2;
		unit->velocity.y /= 2;
		
		if (!(unit->flags & EF_WEIGHTLESS))
		{
			unit->velocity.z = 0;
		}
		
		angle = atan2(self->velocity.y, self->velocity.x);
		self->rotation.x = 90 + toDegrees(angle);
		
		self->thinkTime = 25;
		
		self->oldPosition = self->position;
		
		return;
	}
	*/
	
	unit->lastTargetPosition = self->position;
	
	Vector target;

	switch (rand() % 25)
	{
		case 0:
			target = self->position;
			target.x += Math::rrand(-50, 50);
			target.y += Math::rrand(-50, 50);
			
			if (unit->flags & EF_SWIMS)
			{
				target.z += Math::rrand(-50, 50);
			}
			
			self->velocity = target;
			self->velocity -= self->position;
			self->velocity.normalize();
			self->velocity = self->velocity * 0.25;
			
			faceLocation((Unit*)self, target);
			
			self->thinkTime = Math::rrand(300, 500);

			break;

		case 1:
			self->velocity.x = 0;
			self->velocity.y = 0;
			self->rotation.x = Math::rrand(0, 360);
			self->thinkTime = Math::rrand(100, 300);
			break;

		default:
			self->thinkTime = Math::rrand(1, 10);
			break;
	}
}

void bioMechMoveToTarget()
{
	Unit *unit = (Unit*)self;
	
	enemyMoveToTarget(unit);
	unit->thinkTime = Math::rrand(25, 75);
	faceTarget(unit);
}

void bioMechStrafeAtTarget()
{
	Unit *unit = (Unit*)self;
	unit->action = unit->attack;
	faceTarget(unit);
}

void bioMechHaltAndAttackTarget()
{
	Unit *unit = (Unit*)self;
	
	unit->velocity.x = 0;
	unit->velocity.y = 0;
	unit->thinkTime = Math::rrand(0, 20);
	faceTarget(unit);
	unit->action = unit->attack;
}

void bioMechDodgeAroundTarget()
{
	Unit *unit = (Unit*)self;
	
	float z = unit->velocity.z;
	
	Vector target = unit->target->position;
	target.x += Math::rrand(-50, 50);
	target.y += Math::rrand(-50, 50);
			
	if (unit->flags & EF_SWIMS)
	{
		target.z += Math::rrand(-50, 50);
	}
			
	unit->velocity = target;
	unit->velocity -= unit->position;
	unit->velocity.normalize();
	unit->velocity = unit->velocity * 1.25;
	
	float speed = Math::rrand(150, 400);
	speed /= 750;
			
	faceLocation(unit, target);
	
	unit->velocity.x *= speed;
	unit->velocity.y *= speed;
	
	if ((!(unit->flags & EF_WEIGHTLESS)) && (!(unit->flags & EF_SWIMS)))
	{
		unit->velocity.z = z;
	}
			
	unit->thinkTime = Math::rrand(25, 75);
}

void bioMechJumpAndAttackTarget()
{
	Unit *unit = (Unit*)self;
	
	if (unit->flags & EF_ONGROUND)
	{
		unit->velocity.z = 1.25;
		unit->thinkTime = 50;
		unit->action = &bioMechStrafeAtTarget;
		faceTarget(unit);
	}
}

void bioMechSearch()
{
	Unit *unit = (Unit*)self;
	
	if (unit->target != NULL)
	{
		unit->lastTargetPosition = unit->target->position;
	}
	
	if (unit->flags & EF_NOTARGET)
	{
		generalEnemyLookAround();
		return;
	}
	
	if (!enemySightTarget(unit, 850))
	{
		generalEnemyLookAround();
		return;
	}
	
	// we now know we have a target... Determine whether we'll immediately attack or move around
	
	int r = rand() % 5;
	
	if (r <= 1)
	{
		bioMechMoveToTarget();
		return;
	}
	
	if (!enemySightTarget(unit, 200 + (75 * mission->enemyLevel)))
	{
		generalEnemyLookAround();
		return;
	}
	
	r = Math::rrand(0, 10 + (15 * mission->enemyLevel));
	
	if (r < 10)
	{
		bioMechHaltAndAttackTarget();
	}
	else if (r < 25)
	{
		bioMechDodgeAroundTarget();
	}
	else if (r < 50)
	{
		bioMechStrafeAtTarget();
	}
	else 
	{
		bioMechJumpAndAttackTarget();
	}
}

void bioMechReload()
{
	audio->playSound(SND_PISTOL_RELOAD, CH_ANY);
	self->thinkTime = 100;
	self->action = &bioMechSearch;
}

WeaponDef *getRandomPlayerAmmoType()
{
	if ((game->weapon[1] == NULL) && (game->weapon[2] == NULL))
	{
		return NULL;
	}
	else if ((game->weapon[1] != NULL) && (game->weapon[2] != NULL))
	{
		return (WeaponDef*)game->weapon[1 + rand() % 2]->definition;
	}
	else
	{
		return (WeaponDef*)game->weapon[1]->definition;
	}

	return NULL;
}

void generalUnitFrozen()
{
	// do nothing
	
	self->velocity.x *= 0.98;
	self->velocity.y *= 0.98;
}

int getAdjustedHealthChance(int in, int minimum)
{
	float healthPercent = 100;
	float percent;
	
	// help out the player or teeka if they're in trouble...
	for (Unit *blob = (Unit*)entityManager->blobList.getFirstElement() ; blob != NULL ; blob = (Unit*)blob->next)
	{
		percent = (blob->health / blob->maxHealth) * 100;
		healthPercent = min(percent, healthPercent);
	}
	
	debug(("Minimum Health = %.2f\n", healthPercent));
	
	if (healthPercent >= 25)
	{
		return in;
	}
	
	in = (int)(in * (healthPercent / 15));
	
	Math::limit(&in, minimum, 100);
	
	debug(("getAdjustedHealthChance() - %d (min = %d)\n", in, minimum));
	
	return in;
}

void generalUnitDropWeaponAndItems()
{
	Unit *unit = (Unit*)self;

	if (!(unit->flags & EF_EXPLODES))
	{
		unit->action = &generalBlobDeathSplat;
	}
	else
	{
		unit->action = &generalMechDeathBoom;
	}

	if (unit->dead)
	{
		return;
	}

	if ((unit->currentWeapon->type == WEP_PISTOL) || (unit->currentWeapon->type == WEP_GRENADE) || (unit->currentWeapon->type == WEP_MINI_GRENADES))
	{
		unit->currentWeapon = getRandomPlayerAmmoType();
	}

	if (unit->currentWeapon != NULL)
	{
		Properties weaponProps;
		
		weaponProps.setProperty("classname", unit->currentWeapon->type);
		weaponProps.setProperty("position", unit->position);
		
		Weapon *weapon = addWeapon(&weaponProps);
		WeaponDef *weaponDef = (WeaponDef*)weapon->definition;
		
		float clipSize = weaponDef->clipSize;
		clipSize *= (mission->enemyLevel * 0.65);
		
		int minAmmo = (int)max(1, (clipSize * 0.10));
		int maxAmmo = (int)max(2, (clipSize * 0.15));
		
		weapon->currentAmmo = Math::rrand(minAmmo, maxAmmo);

		throwSpawnedItem(weapon);
		
		if ((rand() % 8) == 0)
		{
			weaponProps.setProperty("classname", "WEP_GRENADE");
			
			weapon = addWeapon(&weaponProps);
			weapon->currentAmmo = 1;
			
			throwSpawnedItem(weapon);
		}
	}

	unit->currentWeapon = NULL;

	if (unit->currentItem != NULL)
	{
		unit->currentItem->position = unit->position;
		
		unit->currentItem->setOwner(NULL);

		throwSpawnedItem(unit->currentItem);

		unit->currentItem = NULL;
	}
	
	int r = rand() % getAdjustedHealthChance(5, 1);
	
	debug(("Cherry chance: %d\n", r));

	if (r == 0)
	{
		Properties itemProps;
		
		itemProps.setProperty("classname", "ITM_CHERRY");
		itemProps.setProperty("position", unit->position);
		
		Item *item = addItem(&itemProps);
		item->itemPower = Math::rrand(1, item->itemPower);

		throwSpawnedItem(item);
	}
	
	r = rand() % getAdjustedHealthChance(50, 3);
	
	debug(("Medipack chance: %d\n", r));
	
	if (r == 0)
	{
		Properties itemProps;
		
		itemProps.setProperty("classname", "ITM_MEDIPACK");
		itemProps.setProperty("position", unit->position);
		
		Item *item = addItem(&itemProps);
		item->itemPower = Math::rrand(10, item->itemPower);

		throwSpawnedItem(item);
	}
}

void generalBlobDie()
{
	Unit *unit = (Unit*)self;
	
	unit->health = 0;
	unit->helpless = 200;
	
	if (unit->contentType & CONTENTS_SOLID)
	{
		self->velocity.z = Math::rrand(50, 100);
		self->velocity.z /= 100;
	}
	
	unit->thinkTime = 0;

	if (unit->liquidLevel != 2)
	{
		audio->playSound(Math::rrand(SND_DEATH1, SND_DEATH4), (unit == player) ? CH_PLAYER : CH_DEATH);
	}
	else if (unit->oxygen != 0)
	{
		audio->playSound(SND_DROWNING, (unit == player) ? CH_PLAYER : CH_DEATH);
	}

	generalUnitDropWeaponAndItems();

	Math::removeBit(&unit->flags, EF_WEIGHTLESS);
}

// ########################### Machine Gun Blob Stuff ################################

void machineGunBlobAttack()
{
	//debug(("machineGunBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &machineGunBlobAttack;

	attackTarget(unit);

	if (--self->custom == 0)
	{
		self->action = &bioMechSearch;
		self->thinkTime = Math::rrand(25, 75);

		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void machineGunBlobPreAttack()
{
	//debug(("machineGunBlobPreAttack()\n"));

	self->custom = Math::rrand(3, 12);
	self->thinkTime = 0;
	self->action = &machineGunBlobAttack;
}

void machineGunBlobInit(Unit *unit)
{
	//debug(("machineGunBlobInit()\n"));

	unit->walk = &bioMechSearch;
	unit->attack = &machineGunBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ########################### Pistol Blob Stuff ################################

void pistolBlobAttack()
{
	//debug(("pistolBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &pistolBlobAttack;

	attackTarget(unit);

	if (--self->custom == 0)
	{
		self->action = &bioMechSearch;
		self->thinkTime = Math::rrand(10, 50);
		
		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void pistolBlobPreAttack()
{
	//debug(("pistolBlobPreAttack()\n"));

	self->custom = Math::rrand(1, 2);
	self->thinkTime = 0;
	self->action = &pistolBlobAttack;
}

void pistolBlobInit(Unit *unit)
{
	//debug(("pistolBlobInit()\n"));

	unit->walk = &bioMechSearch;
	unit->attack = &pistolBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ########################### Shotgun Blob Stuff ################################

void shotGunBlobAttack()
{
	//debug(("shotGunBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &shotGunBlobAttack;

	attackTarget(unit);

	if (--self->custom == 0)
	{
		self->action = &bioMechSearch;
		self->thinkTime = Math::rrand(25, 75);
		
		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void shotGunBlobPreAttack()
{
	self->custom = 1;
	self->thinkTime = 0;
	self->action = &shotGunBlobAttack;
}

void shotGunBlobInit(Unit *unit)
{
	unit->walk = &bioMechSearch;
	unit->attack = &shotGunBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ############################ Grenade Blob Stuff ###################################

void grenadeBlobAttack()
{
	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	attackTarget(unit);

	if (--unit->custom == 0)
	{
		self->thinkTime = Math::rrand(100, 200);
		self->action = &bioMechSearch;
		
		if ((rand() % 5) == 0)
		{
			self->thinkTime = 100;
			self->action = &bioMechReload;
		}
	}
}

void grenadeBlobPreAttack()
{
	if ((rand() % 4) > 1)
	{
		((Unit*)self)->currentWeapon = &entityManager->weaponDef[WEP_GRENADE];
		self->custom = Math::rrand(1, 3);
		self->action = grenadeBlobAttack;
	}
	else
	{
		((Unit*)self)->currentWeapon = &entityManager->weaponDef[WEP_PISTOL];
		self->custom = Math::rrand(1, 2);
		self->action = pistolBlobAttack;
	}
}

void grenadeBlobInit(Unit *unit)
{
	unit->walk = &bioMechSearch;
	unit->attack = &grenadeBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ########################## Plasma pistol blob ###################

void plasmaPistolBlobInit(Unit *unit)
{
	//debug(("pistolBlobInit()\n"));

	unit->walk = &bioMechSearch;
	unit->attack = &pistolBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}

// ########################### Pistol Blob Stuff ################################

void aquaBlobAttack()
{
	//debug(("pistolBlobAttack()\n"));

	Unit *unit = (Unit*)self;

	faceTarget(unit);

	if (unit->reload > 0)
	{
		return;
	}

	self->thinkTime = 0;
	self->action = &aquaBlobAttack;

	attackTarget(unit);

	self->action = &bioMechSearch;
	self->thinkTime = Math::rrand(100, 200);
	
	if ((rand() % 5) == 0)
	{
		self->thinkTime += 100;
		self->action = &bioMechReload;
	}
}

void aquaBlobPreAttack()
{
	//debug(("pistolBlobPreAttack()\n"));

	self->thinkTime = 0;
	self->action = &aquaBlobAttack;
}

void aquaBlobInit(Unit *unit)
{
	//debug(("pistolBlobInit()\n"));

	unit->walk = &bioMechSearch;
	unit->attack = &aquaBlobPreAttack;
	unit->die = &generalBlobDie;

	unit->thinkTime = Math::rrand(0, 50);

	unit->action = unit->walk;
}
